<?php

namespace App\Controller;

use App\Entity\CaseItinerary;
use App\Entity\CaseLocation;
use App\Entity\Location;
use App\Entity\MemberCase;
use App\Entity\Messages;
use App\Entity\User;
use App\Libs\Breadcrumb;
use App\Libs\Libs;
use App\Libs\NavList;
use DateTime;
use Doctrine\ORM\EntityManagerInterface;
use Symfony\Bundle\FrameworkBundle\Controller\AbstractController;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\HttpFoundation\Session\Session;
use Symfony\Component\Routing\Attribute\Route;
use Symfony\Component\Security\Http\Attribute\CurrentUser;
use Symfony\UX\Map\InfoWindow;
use Symfony\UX\Map\Map;
use Symfony\UX\Map\Marker;
use Symfony\UX\Map\Point;
use Symfony\UX\Map\Polyline;

class ItineraryController extends AbstractController
{
    private array $msgs = [];

    private int $notificationCount = 0;

    public function __construct(
        private EntityManagerInterface $entityManager,
        private array $navLinks = []
    ) {
        $this->navLinks = NavList::LIST;
    }

    #[Route('/itinerary/map', name: 'app_map_itinerary')]
    public function mapItinerary(Request $request, #[CurrentUser()] ?User $user): Response
    {
        $this->msgs = $this->entityManager->getRepository(Messages::class)->getUnreadMessages($user);
        $this->notificationCount = $this->entityManager->getRepository(Messages::class)->getUnreadMessageCount($user);

        $itineraries = $this->entityManager->getRepository(CaseItinerary::class)->findBy([
            'memberCase' => $request->getPayload()->get('caseId'),
            'date' => new DateTime($request->getPayload()->get('caseDate'))
        ]);

        $map = new Map('default');
        $map->center(new Point(39.768502, -86.157918))
            ->zoom(9)
        ;

        $total_distance = 0;
        $total_duration = 0;

        foreach ($itineraries as $itinerary) {
            /** @var CaseItinerary $itinerary */
            $map->addPolyline(new Polyline(
                points: $itinerary->getGPSPolyLines(),
                infoWindow: new InfoWindow(
                    content: $itinerary->getMemberCase()->getCaseName()
                )
            ));

            $total_distance += $itinerary->getDistance();
            $total_duration += $itinerary->getDuration()->s;
        }

        return $this->render(
            'internal/cases/itinerary/map.html.twig',
            array_merge(
                $this->navLinks,
                [
                    'breadcrumbs' => [
                        new Breadcrumb($this->generateUrl('app_my_cases'), 'My Cases'),
                    ],
                    'notifications' => $this->msgs,
                    'notificationCount' => $this->notificationCount,
                    'map' => $map,
                    'total_distance' => $total_distance,
                    'total_duration' => $total_duration,
                ]
            )
        );
    }

    #[Route('/api/get-case-locations/{caseId}', name: 'get_case_locations')]
    public function createItinerary(string $caseId): Response
    {
        $case = $this->entityManager->getRepository(MemberCase::class)->find($caseId);
        $cls = $this->entityManager->getRepository(CaseLocation::class)->getCaseLocations($case);

        $locations = [];
        foreach ($cls as $cl) {
            /** @var CaseLocation $cl */
            $locations[] = $cl->getLocation();
        }

        return $this->json($locations);
    }

    #[Route('/api/add-location-to-itinerary', name: 'add_location_to_itinerary')]
    public function addLocationToItinerary(Request $request, Session $session): Response
    {
        $case = $this->entityManager->getRepository(MemberCase::class)->find($request->getPayload()->get('caseId'));
        $origin = $this->entityManager->getRepository(Location::class)->find($request->getPayload()->get('origin'));
        $destination = $this->entityManager->getRepository(Location::class)->find($request->getPayload()->get('destination'));
        $departure = $request->getPayload()->get('departure');
        $arrival = $request->getPayload()->get('arrival');
        $caseMileage = (bool) $request->getPayload()->get('caseMileage');
        $date = new DateTime($request->getPayload()->get('date'));

        $route = Libs::getRouteDistance($origin, $destination);

        if (!$route) {
            return $this->json(['success' => false, 'message' => 'No route found']);
        }

        $ci = new CaseItinerary();
        $ci->setMemberCase($case)
           ->setDate($date)
           ->setCaseMileage($caseMileage)
           ->setOriginLocation($origin)
           ->setDestLocation($destination)
           ->setDeparture(new \DateTimeImmutable($departure))
           ->setArrival(new \DateTimeImmutable($arrival))
           ->setDistance($route->getDistance())
           ->setDuration($route->getDuration())
           ->setGpsRoute($route->getGeometry())
        ;

        $this->entityManager->persist($ci);
        $this->entityManager->flush();

        $session->getFlashBag()->add(
            'success',
            'Location added to itinerary'
        );
        return $this->json(['success' => true, 'message' => 'Location added to itinerary']);
    }
}
